package client.controller;

import client.gui.UpdateGui;
import client.model.Client;
import client.model.exception.ConnectException;

/**
 * User: sumb
 * Date: 5/27/11
 * Time: 11:53 PM
 */
public final class Controller
{
    private final static Client client = new Client();

    public void connect(final String nick, final String server, final String port) throws ConnectException
    {
        check(nick);
        check(server);
        check(port);

        int intPort;
        try
        {
            intPort = Integer.parseInt(port);
        }
        catch(final NumberFormatException ex)
        {
            throw new ConnectException("Bad port");
        }

        client.connect(server, intPort);
        client.sendName(nick);
    }

    public void setGui(final UpdateGui gui)
    {
        check(gui);
        client.setGui(gui);
    }

    public void getMessages()
    {
        client.getMessages();
    }

    public void sendMessage(final String message)
    {
        check(message);
        client.sendMessage(message);
    }

    public void getMembersList()
    {
        client.getMembersList();
    }

    private void check(final Object obj)
    {
        if(null == obj)
        {
            throw new IllegalArgumentException();
        }
    }

    public void getLastMessages()
    {
        client.getLastMessages();
    }
}
