package client.gui;

import client.controller.Controller;
import client.gui.utilities.SpringUtilities;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.HashSet;
import java.util.Set;

/**
 * User: sumb
 * Date: 5/27/11
 * Time: 11:14 PM
 */
public final class ChatFrame implements UpdateGui
{
    private static final int BORDER_SIZE = 5;
    private static final int MAXIMUM_SEND_PANEL_WIDTH = 2000;
    private static final int MAXIMUM_SEND_PANEL_HEIGHT = 20;
    private static final int MAXIMUM_MESSAGE_FIELD_WIDTH = 2000;
    private static final int MAXIMUM_MESSAGE_FIELD_HEIGHT = 10;
    private static final int LIST_SCROLLER_WIDTH = 200;
    private static final int LIST_SCROLLER_HEIGHT = 500;
    private static final int CHAT_AREA_SCROLLER_WIDTH = 500;
    private static final int CHAT_AREA_SCROLLER_HEIGHT = 500;

    private static final int MAX_CHARS_IN_MESSAGE = 10;

    private static final JFrame frame = new JFrame();
    private static final JPanel chatPanel = new JPanel(new SpringLayout());
    private static final JPanel sendPanel = new JPanel(new SpringLayout());
    private static final JTextArea chatArea = new JTextArea();
    private static final JTextField messageField = new JTextField();
    private static final JList membersJList = new JList();
    private Set<String> onlineList;
    private static final JButton sendButton = new JButton("Send");

    private static final JScrollPane chatAreaScroller = new JScrollPane(chatArea);
    private static final JScrollPane membersListScroller = new JScrollPane(membersJList);

    private Controller controller;

    public void printMessage(final String message)
    {
        chatArea.append(message + "\n");
        chatArea.setColumns(chatArea.getColumns() + 1);
        chatArea.setRows(chatArea.getRows() + 1);
        messageField.setText("");
        messageField.requestFocus();
    }

    private void sendMessage(final String message)
    {
        if(!"".equals(messageField.getText()))
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    controller.sendMessage(messageField.getText());
                }
            });
        }
    }

    private void prepareElements()
    {
        /* chatArea */
        chatArea.setEditable(false);
        chatAreaScroller.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        chatAreaScroller.setBorder(
                BorderFactory.createCompoundBorder(
                        BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder("Chat"),
                                BorderFactory.createEmptyBorder(BORDER_SIZE,BORDER_SIZE,BORDER_SIZE,BORDER_SIZE)),
                        chatAreaScroller.getBorder()));

        /* membersJList */
        membersJList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        membersJList.setLayoutOrientation(JList.HORIZONTAL_WRAP);

        membersListScroller.setVerticalScrollBarPolicy(
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        membersListScroller.setBorder(
                BorderFactory.createCompoundBorder(
                        BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder("Members"),
                                BorderFactory.createEmptyBorder(BORDER_SIZE,BORDER_SIZE,BORDER_SIZE,BORDER_SIZE)),
                        membersListScroller.getBorder()));

        /* sendButton */
        sendButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                sendMessage(messageField.getText());
            }
        });

        messageField.setDocument(new PlainDocument()
        {
            public void insertString(final int offset, final String str, final AttributeSet attr)
                    throws BadLocationException
            {
                if (str == null)
                {
                    return;
                }
                if ((getLength() + str.length()) <= MAX_CHARS_IN_MESSAGE)
                {
                    super.insertString(offset, str, attr);
                }
            }
        });

        messageField.addKeyListener(new KeyListener()
        {
            public void keyTyped(KeyEvent keyEvent)
            {
                if('\n' == keyEvent.getKeyChar())
                {
                    sendMessage(messageField.getText());
                }
            }

            public void keyPressed(KeyEvent keyEvent) {}

            public void keyReleased(KeyEvent keyEvent) {}
        });
    }

    private void setSizes()
    {
        sendPanel.setMaximumSize(new Dimension(MAXIMUM_SEND_PANEL_WIDTH, MAXIMUM_SEND_PANEL_HEIGHT));
        messageField.setMaximumSize(new Dimension(MAXIMUM_MESSAGE_FIELD_WIDTH, MAXIMUM_MESSAGE_FIELD_HEIGHT));
        membersListScroller.setMaximumSize(new Dimension(LIST_SCROLLER_WIDTH, LIST_SCROLLER_HEIGHT));
        membersListScroller.setPreferredSize(new Dimension(LIST_SCROLLER_WIDTH, LIST_SCROLLER_HEIGHT));
        chatAreaScroller.setPreferredSize(new Dimension(CHAT_AREA_SCROLLER_WIDTH, CHAT_AREA_SCROLLER_HEIGHT));
    }

    void makeFrame()
    {
        prepareElements();
        setSizes();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.Y_AXIS));
        frame.add(chatPanel);
        frame.add(sendPanel);
        chatPanel.add(chatAreaScroller);
        chatPanel.add(membersListScroller);
        sendPanel.add(messageField);
        sendPanel.add(sendButton);

        SpringUtilities.makeCompactGrid(sendPanel, 1);
        SpringUtilities.makeCompactGrid(chatPanel, 1);

        frame.pack();
        frame.setVisible(true);
    }


    public ChatFrame(final Controller _controller)
    {
        controller = _controller;
        controller.setGui(this);
        makeFrame();
        controller.getLastMessages();
        controller.getMessages();
        getCoreMembersList();
    }

    /* set members list */

    void getCoreMembersList()
    {
        controller.getMembersList();
    }

    public void globalSetMembersList(final Set<String> list)
    {
        onlineList = list;
        membersJList.setListData(list.toArray());
    }

    public void userIsOffline(String user)
    {
        if(null != onlineList)
        {
            onlineList.remove(user);
            membersJList.setListData(onlineList.toArray());
        }
    }

    public void userIsOnline(String user)
    {
        if(null == onlineList)
        {
            onlineList = new HashSet<String>();
        }

        onlineList.add(user);
        membersJList.setListData(onlineList.toArray());
    }
}
