package client.gui;

import client.controller.Controller;
import client.gui.utilities.SpringUtilities;
import client.model.exception.ConnectException;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * User: sumb
 * Date: 5/27/11
 * Time: 11:19 PM
 */
public final class LoginFrame implements Runnable
{
    private static final int GRID_NUMBER = 3;
    private static final int BUTTON_WIDTH = 100;
    private static final int BUTTON_HEIGHT = 30;

    /* main elements */
    private static final JLabel welcomeLabel = new JLabel(
            "<html>" +
             "<h2>Welcome to chat</h2>" +
            "<html>");
    private static final JLabel nickLabel = new JLabel("Nickname: ");
    private static final JLabel serverLabel = new JLabel("Server: ");
    private static final JLabel portLabel = new JLabel("Port: ");
    private static final JLabel errorLabel = new JLabel();

    private static final JTextField nickField = new JTextField(15);
    private static final JTextField serverField = new JTextField(30);
    private static final JTextField portField = new JTextField(5);

    private static final JButton connectButton = new JButton("Connect");
    private static final JButton exitButton = new JButton("Exit");

    /* main frame and layouts */
    private static final JFrame frame = new JFrame();
    private static final JPanel welcomePanel = new JPanel();
    private static final JPanel loginPanel = new JPanel(new SpringLayout());
    private static final JPanel buttonPanel = new JPanel(new FlowLayout());
    private static final JPanel errorPanel = new JPanel();

    private static final Controller controller = new Controller();

    public LoginFrame()
    {
    }

    private void setError(final String error)
    {
        errorLabel.setText(
                "<html>" +
                 "<b><font color=red>" + error + "</font></b>" +
                "</html>"
        );

        frame.pack();
    }

    private void addListenersToButtons()
    {
        exitButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                frame.dispose();
            }
        });

        connectButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent actionEvent)
            {
                final String nick = nickField.getText();
                final String server = serverField.getText();
                final String port = portField.getText();

                if("".equals(nick) ||
                   "".equals(server) ||
                   "".equals(port))
                {
                    setError("Enter all fields!");
                }
                else
                {
                    new Thread(new Runnable() {
                        public void run() {
                            try
                            {
                                controller.connect(nick, server, port);
                            }
                            catch (ConnectException e)
                            {
                                setError(e.getMessage());
                                return;
                            }

                            frame.dispose();
                            new ChatFrame(controller);
                        }
                    }).start();
                }
            }
        });
    }

    private void makeFrame()
    {

        /* remove */
        nickField.setText("nickname");
        serverField.setText("localhost");
        portField.setText("55555");
        /* end remove */

        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setTitle("Chat");
        frame.setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.Y_AXIS));
        frame.add(welcomePanel);
        frame.add(loginPanel);
        frame.add(errorPanel);
        frame.add(buttonPanel);

        welcomePanel.add(welcomeLabel);

        loginPanel.add(nickLabel);
        loginPanel.add(nickField);
        loginPanel.add(serverLabel);
        loginPanel.add(serverField);
        loginPanel.add(portLabel);
        loginPanel.add(portField);

        SpringUtilities.makeCompactGrid(loginPanel, GRID_NUMBER);

        addListenersToButtons();
        connectButton.setPreferredSize(new Dimension(BUTTON_WIDTH,BUTTON_HEIGHT));
        exitButton.setPreferredSize(new Dimension(BUTTON_WIDTH,BUTTON_HEIGHT));
        buttonPanel.add(connectButton);
        buttonPanel.add(exitButton);

        errorPanel.add(errorLabel);

        frame.pack();
        frame.setVisible(true);

    }

    public void run()
    {
        makeFrame();
    }
}
