package client.model.executor;

import client.gui.UpdateGui;
import client.model.exception.ConnectException;
import client.model.exception.ParseException;
import client.model.executor.messagesProcessing.GeneralMessageProcessing;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.util.HashMap;

/**
 * User: sumb
 * Date: 5/30/11
 * Time: 5:40 AM
 */
public final class Executor
{
    private static final int MESSAGE_CLASS_NUMBER = 0;
    private static final int PROCESS_CLASS_NUMBER = 1;
    private static final int COMMANDS_NUMBER = 2;

    private UpdateGui gui = null;
    private final HashMap<String, GeneralMessageProcessing> processMap = new HashMap<String, GeneralMessageProcessing>();


    public void processMessage(final Object object) throws ConnectException
    {
        if(null == object)
        {
            throw new IllegalArgumentException("Arg can't be null");
        }

        if(!processMap.containsKey(object.getClass().getName()))
        {
            throw new IllegalArgumentException("Config file don't have class: " + object.getClass().getName());
        }

        processMap.get(object.getClass().getName()).exec(gui, object);
    }

    public void initExecutor(final Reader _reader) throws ParseException
    {
        BufferedReader reader = new BufferedReader(_reader);

        String line;
        try
        {
            line = reader.readLine();

            while(null != line)
            {
                String[] processClass = line.split("[ ]");

                if(COMMANDS_NUMBER != processClass.length)
                {
                    throw new ParseException("Can't parse string: " + line);
                }

                Class commandClass = Class.forName(processClass[PROCESS_CLASS_NUMBER]);
                GeneralMessageProcessing commandObject = (GeneralMessageProcessing) commandClass.newInstance();
                processMap.put(processClass[MESSAGE_CLASS_NUMBER], commandObject);

                line = reader.readLine();
            }
        }
        catch (ClassNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (InstantiationException e)
        {
            e.printStackTrace();
        }
        catch (IllegalAccessException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    public void setGui(final UpdateGui _gui)
    {
        gui = _gui;
    }

}
