package server;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * User: sumb
 * Date: 5/31/11
 * Time: 6:40 PM
 */
final class ClientSocket
{
    private Socket clientSocket;
    private ObjectInputStream iStream = null;
    private ObjectOutputStream oStream = null;
    private String name = null;

    public ClientSocket(final ServerSocket serverSocket) throws IOException
    {
        if(null == serverSocket)
        {
            throw new IllegalArgumentException("serverSocket == null");
        }

        clientSocket = serverSocket.accept();

        try
        {
            iStream = new ObjectInputStream(clientSocket.getInputStream());
        }
        catch(final IOException e)
        {
            e.printStackTrace();
            return;
        }

        try
        {
            oStream = new ObjectOutputStream(clientSocket.getOutputStream());
        }
        catch(final IOException e)
        {
            iStream.close();
            clientSocket.close();
            e.printStackTrace();
        }
    }

    public String getName() throws ClassNotFoundException, IOException
    {
        if(null != name)
        {
            return name;
        }

        name = (String)iStream.readObject();

        return name;
    }

    public void sendMessage(Object message) throws IOException
    {
        if(null != message)
        {
            oStream.writeObject(message);
        }
    }

    public Object getMessage() throws ClassNotFoundException, IOException
    {
        return iStream.readObject();
    }

    public String getAddress()
    {
        return clientSocket.getInetAddress().getHostAddress() +
                ":" +
                clientSocket.getInetAddress().getHostName();
    }

    public void close() throws IOException
    {
        iStream.close();
        oStream.close();
        clientSocket.close();
    }
}
