package server;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * User: sumboid
 * Date: 5/16/11
 * Time: 7:20 PM
 */
public final class Main
{
    private static final int SERVER_NUMBER = 0;
    private static final int PORT_NUMBER = 1;
    private static final int ARGS_NUMBER = 2;

    private static final int MIN_NUMBERS_OF_THREAD = 1;
    private static final int MAX_NUMBERS_OF_THREAD = 10;
    private static final int SIZE_OF_QUEUE = 10;
    private static final int TIME = 0;


    private static final Logger logger = Logger.getLogger(Main.class);

    public static void main(String args[])
    {
        Server server;

        if(0 == args.length)
        {
            try
             {
                 logger.info("Starting server on 127.0.0.1:55555");
                 server = new Server("127.0.0.1", 55555);
             }
             catch (IOException e)
             {
                 e.printStackTrace();
                 logger.info("!! Catch IOException. Stack trace:");
                 logger.info(e.getStackTrace());
                 return;
             }

        }
        else
        {
            if(ARGS_NUMBER  != args.length)
            {
                System.out.println("Usage: server,jar ip port");
                return;
            }

            logger.info("-- Start");

            int intPort;
            try
            {
                intPort = Integer.parseInt(args[PORT_NUMBER]);
            }
            catch(final NumberFormatException ex)
            {
                logger.info("!! Bad port: " + args[PORT_NUMBER]);
                System.err.println("Bad port!");
                return;
            }

            try
            {
                logger.info("Starting server on " + args[SERVER_NUMBER] + ":" + args[PORT_NUMBER]);
                server = new Server(args[SERVER_NUMBER], intPort);
            }
            catch (IOException e)
            {
                e.printStackTrace();
                logger.info("!! Catch IOException. Stack trace:");
                logger.info(e.getStackTrace());
                return;
            }
        }

        ArrayBlockingQueue<Runnable> queue = new ArrayBlockingQueue<Runnable>(SIZE_OF_QUEUE);

        ThreadPoolExecutor pool = new ThreadPoolExecutor(MIN_NUMBERS_OF_THREAD,
                                                         MAX_NUMBERS_OF_THREAD,
                                                         TIME,
                                                         TimeUnit.SECONDS,
                                                         queue,
                                                         new RejectedExecutionHandler()
        {
            public void rejectedExecution(Runnable r, ThreadPoolExecutor executor)
            {
                if (!executor.isShutdown())
                {
                    try
                    {
                        executor.getQueue().put(r);
                    }
                    catch (InterruptedException ex)
                    {
                        logger.info("!! Catch InterruptException. Stack trace:");
                        logger.info(ex.getStackTrace());
                        ex.printStackTrace();
                    }
                }
            }
        });

        while(true) //It is normal, because server don't stop
        {
            pool.execute(server);
        }
    }
}